#include <iostream>
#include <fstream>
#include <math.h>

using namespace std;

ifstream fin("6_kalitkak.in");

int lepesekSzama = 0;// 

int const maxDim = 100001;

// 1. Változat:

int nyitva_V1(int n){             
// a nyitva maradt kalitkák számának meghatározása
// alkalmazzuk a bináris keresés algoritmusát n négyzetgyökének kiszámítására: O(log n)
	
	int bal = 1;
	int jobb = n / 2;
		
	while(bal < jobb){
		int k = ((bal + jobb) / 2);
		if(k * k >= n)
			jobb = k;
		else
			bal = k + 1;
        lepesekSzama++;
	}
	if(bal * bal <= n)		
		return bal;
	else
		return bal - 1;
}

// 2. Változat:

double abszErtek(double a){
    if (a < 0)
        return -a;
    return a;
}

int nyitva_V2(double n){                  
// közelítő algoritmus n négyzetgyökének kiszámítására: O(log n)

    const double eps = 0.00001;
    double x = (double)n;
    double y = (x + 1) / 2;
    double absV = abszErtek(x - y);
    while (absV >= eps){
        x = y;
        y = double(1) / double(2) * (x + n / x);
        absV = abszErtek(x - y);
        lepesekSzama++;
    }   //while
    return (int)x;
}

// 3. Változat:

int nyitva_V3(int n){             
// megszámoljuk az n-nél kisebb négyzetszámokat: O(sqrt(n))

int nyitottSz = 0;

    for(int i = 1; i * i <= n; i++){
        lepesekSzama++;
        nyitottSz++;
    }
    return nyitottSz;
}

// 4. Változat:

int nyitva_V4(int n){
// aritmetikai függvény
// memória: O(1), idő: O(sqrt(n))

int x = 0;
int z = 1;
  while(z <= n){
    x++;
    z = z + 2 * x;
    z++;
    lepesekSzama++;
  }
  return x; 
}

// 5. Változat:

bool negyzetSz(int x){
// true-t, térítünk, ha x négyzetszám

    for (int d = 1; d * d <= x; d++){
        if (d * d == x)
            return true;
    }
    return false;
}

int nyitva_V5(int n){
// minden n-nél kisebb vagy n-nel egyenlő számról eldöntjük,
// hogy négyzetszám-e vagy sem: O(n * sqrt(n))

int nyitottSz = 0;

    for(int i = 1; i <= n; i++){
        if(negyzetSz(i))
            nyitottSz++;
        lepesekSzama++;
    }
    return nyitottSz;
}

// 6. Változat

int osztokSz(int x){
    int k = 1;
    for (int i = 2; i <= x; i++){
        if (x % i == 0)
            k++;
    }
    return k;
}

int nyitva_V6(int n){
// megszámoljuk az nél kisebb számokat, amelyeknek páratlan darabszámú osztójuk van
// O(n^2)

int nyitottSz = 0;
    for (int i = 1; i <= n; i++){
        lepesekSzama++;
        if (osztokSz(i) % 2 == 1)
            nyitottSz++;
    }
    return nyitottSz;
}

// 7. Változat

int nyitva_V7(int n){               
// szimulálás

    const int MAX = 10001;
    bool ajtok[MAX];
    for (int i = 1; i <= n; i++){
        ajtok[i] = true;
        lepesekSzama++;
    }
    for (int k = 2; k <= n; k++){
        for (int i = k; i <= n; i = i + k){
            lepesekSzama++; 
            if (ajtok[i])
                ajtok[i] = false;
            else
                ajtok[i] = true;
        }
    }
    int nyitottSz = 0;
    for (int i = 1; i <= n; i++)
    if (ajtok[i]){
        nyitottSz++;
        lepesekSzama++;
    }
    return nyitottSz;
}

// 8. Változat

int nyitva_V8(int n){
// 2. szimulálás
// O(n * log n)

bool nyitott[maxDim];

    for(int i = 1; i <= n; i++){
        lepesekSzama++;
        nyitott[i] = true;
    }

    for(int i = 2; i <= n; i++)
        for(int j = i; j <= n; j+=i){  
            nyitott[j] = !nyitott[j];
            lepesekSzama++;
        }
    
    int nyitottSz = 0;
    for(int i = 1; i <= n; i++){
        if(nyitott[i]){
            nyitottSz++;
            lepesekSzama++;
        }
    }
    return nyitottSz;
}

int main(){

    int n;
    cout << "Kalitkak szama = "; cin >> n;
    cout << "V1 - binaris kereses: " << endl;
	cout << "Nyitva maradt kalitkak szama = " << nyitva_V1(n) << endl;
    cout << "Lepesek szama:" << lepesekSzama << endl << endl;

    lepesekSzama = 0;
    cout << "V2 - kozelito algoritmus: " << endl;
	cout << "Nyitva maradt kalitkak szama = " << nyitva_V2(n) << endl;
    cout << "Lepesek szama:" << lepesekSzama << endl << endl;

    lepesekSzama = 0;
    cout << "V3 - for a negyzetgyokig: " << endl;
	cout << "Nyitva maradt kalitkak szama = " << nyitva_V3(n) << endl;
    cout << "Lepesek szama:" << lepesekSzama << endl << endl;

    lepesekSzama = 0;
    cout << "V4 - aritmetikai fuggveny: " << endl;
    cout << "Nyitva maradt kalitkak szama = " << nyitva_V4(n) << endl;
    cout << "Lepesek szama:" << lepesekSzama << endl << endl;

	lepesekSzama = 0;
    cout << "V5 - n-nel kisebb vagy n-nel egyenlo szamok kozotti negyzetszamok: " << endl;
	cout << "Nyitva maradt kalitkak szama = " << nyitva_V5(n) << endl;
    cout << "Lepesek szama:" << lepesekSzama << endl << endl;

    lepesekSzama = 0;
    cout << "V6 - paratlan darabszamu osztokkal rendelkezo szamok: " << endl;
	cout << "Nyitva maradt kalitkak szama = " << nyitva_V6(n) << endl;
    cout << "Lepesek szama:" << lepesekSzama << endl << endl;

    lepesekSzama = 0;
    cout << "V7 - szimulalas 1: " << endl;
    cout << "Nyitva maradt kalitkak szama = " << nyitva_V7(n) << endl;
    cout << "Lepesek szama:" << lepesekSzama << endl << endl;

	lepesekSzama = 0;
    cout << "V8 - szimulalas 2: " << endl;
    cout << "Nyitva maradt kalitkak szama = " << nyitva_V8(n) << endl;
    cout << "Lepesek szama:" << lepesekSzama << endl << endl;
}
